import urllib2
import urllib
import sys

from xml.sax import parseString
from StringIO import StringIO

from py_w3c.multipart import Multipart
from py_w3c.handler import ValidatorHandler
from py_w3c.exceptions import ValidationFault
from py_w3c import __version__

try:
    unicode       # Py2
except NameError:  # Py3
    unicode = str

VALIDATOR_URL = 'http://validator.w3.org/check'


class HTMLValidator(object):
    def __init__(self, validator_url=VALIDATOR_URL, charset=None, doctype=None, verbose=False):
        self.validator_url = validator_url
        self.result = None
        self.uri = ''
        self.uploaded_file = ''
        self.output = 'soap12'
        self.charset = charset
        self.doctype = doctype
        self.errors = []
        self.warnings = []
        self.verbose = verbose

    def _validate(self, url, headers=None, post_data=None):
        '''
        Sends request to the validator, if post_data is not empty, sends
        POST request, otherwise sends GET request. Returns True if validation
        occurs, otherwise raises exception.
        '''
        if not headers:
            headers = {}
        req = urllib2.Request(url, headers=headers, data=post_data)
        resp = urllib2.urlopen(req)
        self._process_response(resp.read())
        return True

    def validate(self, uri):
        '''
        Validates by uri.
        '''
        get_data = {'uri': uri, 'output': self.output}
        if self.charset:
            get_data['charset'] = self.charset
        if self.doctype:
            get_data['doctype'] = self.doctype
        get_data = urllib.urlencode(get_data)
        return self._validate(self.validator_url + '?' + get_data)

    def validate_file(self, filename_or_file, name='file'):
        '''
        Validates file by filename or file content.
        '''
        m = Multipart()
        m.field('output', self.output)
        if self.doctype:
            m.field('doctype', self.doctype)
        if self.charset:
            m.field('charset', self.charset)
        if isinstance(filename_or_file, (str, unicode)):
            with open(filename_or_file, "r") as w:
                content = w.read()
        elif isinstance(filename_or_file, file):
            content = filename_or_file.read()
        elif isinstance(filename_or_file, StringIO):
            content = filename_or_file.getvalue()
        else:
            raise Exception(
                'File name or file only. Got %s instead' % type(filename_or_file))
        m.file('uploaded_file', name, content, {'Content-Type': 'text/html'})
        ct, body = m.get()
        return self._validate(self.validator_url, headers={'Content-Type': ct}, post_data=body)

    def validate_fragment(self, fragment):
        '''
        Validates fragment. Full html fragment only.
        '''
        post_data = {'fragment': fragment, 'output': self.output}
        if self.doctype:
            post_data['doctype'] = self.doctype
        if self.charset:
            post_data['charset'] = self.charset
        post_data = urllib.urlencode(post_data)
        return self._validate(self.validator_url, post_data=post_data)

    def _process_response(self, response):
        val_handler = ValidatorHandler()
        parseString(response, val_handler)
        if val_handler.fault_occured:
            raise ValidationFault('Fault occurs. %s' % val_handler.fault_message)
        if self.verbose:
            print 'Errors: %s' % len(self.errors)
            print 'Warnings: %s' % len(self.warnings)
        self.result = val_handler
        self.warnings = val_handler.warnings
        self.errors = val_handler.errors


def main(argv=None):
    usage = '  Usage: \n    w3c_validate http://yourdomain.org'
    if argv is None:
        argv = sys.argv
    if len(argv) != 2:
        print usage
        sys.exit(2)
    if argv[1] in ('-v', '--version'):
        print __version__
        sys.exit(0)
    val = HTMLValidator(verbose=False)
    val.validate(argv[1])
    print '---warnings---(%s)' % len(val.warnings)
    for warning in val.warnings:
        print 'line:%s; col:%s; message:%s'\
            % (warning.get('line'), warning.get('col'), warning.get('message'))
    print '---errors---(%s)' % len(val.errors)
    for error in val.errors:
        print 'line:%s; col:%s; message:%s'\
            % (error.get('line'), error.get('col'), error.get('message'))
    sys.exit(0)



#!/usr/bin/python
'''
Handler for W3C validation api response.
'''

from xml.sax import handler


class ValidatorHandler(handler.ContentHandler):
    def __init__(self):
        self.tags = {}
        self.doctype = ''
        self.charset = ''
        self.validity = ''
        self.errors = []
        self.warnings = []
        self.last_error = {}
        self.last_warning = {}
        self.errorcount = 0
        self.warningcount = 0
        self.fault_occured = False
        self.fault_message = ""

    def startElement(self, name, attrs):
        if ':' in name:
            namespace, localname = name.split(':')
        else:
            localname = name
        self.tags.update({'in_%s' % localname: True})

    def characters(self, content):
        if self.tags.get('in_doctype'):
            self.doctype = content
        if self.tags.get('in_charset'):
            self.charset = content
        if self.tags.get('in_validity'):
            self.validity = content

        if self.tags.get('in_Envelope') and self.tags.get('in_Body') and self.tags.get('in_Fault') and self.tags.get('in_errordetail'):
            self.fault_occured = True
        self.fault_message = '%s%s' % (self.fault_message, content)

        if self.tags.get('in_errors') and self.tags.get('in_errorcount'):
            self.errorcount = int(content)

        if self.tags.get('in_errorlist') and self.tags.get('in_error'):
            if self.tags.get('in_line'):
                self.last_error.update({'line': content})
            elif self.tags.get('in_col'):
                self.last_error.update({'col': content})
            elif self.tags.get('in_source'):
                self.last_error.update(
                    {'source': '%s%s' % (self.last_error.get('source', ''), content)})
            elif self.tags.get('in_explanation'):
                self.last_error.update(
                    {'explanation': '%s%s' % (self.last_error.get('explanation', ''), content)})
            elif self.tags.get('in_message'):
                self.last_error.update(
                    {'message': '%s%s' % (self.last_error.get('message', ''), content)})
            elif self.tags.get('in_messageid'):
                self.last_error.update({'messageid': content})

        if self.tags.get('in_warnings') and self.tags.get('in_warningcount'):
            self.warningcount = int(content)

        if self.tags.get('in_warninglist') and self.tags.get('in_warning'):
            if self.tags.get('in_line'):
                self.last_warning.update({'line': content})
            elif self.tags.get('in_col'):
                self.last_warning.update({'col': content})
            elif self.tags.get('in_source'):
                self.last_warning.update(
                    {'source': '%s%s' % (self.last_error.get('source', ''), content)})
            elif self.tags.get('in_explanation'):
                self.last_warning.update(
                    {'explanation': '%s%s' % (self.last_error.get('explanation', ''), content)})
            elif self.tags.get('in_message'):
                self.last_warning.update(
                    {'message': '%s%s' % (self.last_error.get('message', ''), content)})
            elif self.tags.get('in_messageid'):
                self.last_warning.update({'messageid': content})

    def endElement(self, name):
        if ':' in name:
            namespace, localname = name.split(':')
        else:
            localname = name
        self.tags.update({'in_%s' % localname: False})

        if localname == 'error':
            self.errors.append(self.last_error.copy())
            self.last_error = {}



#!/usr/bin/python
'''
Classes for using multipart form data from Python, which does not (at the
time of writing) support this directly.
 
To use this, make an instance of Multipart and add parts to it via the factory
methods field and file.  When you are done, get the content via the get method.
 
@author: Stacy Prowell (http://stacyprowell.com)
'''

# example
# >>> from multipart import Multipart
# >>> m = Multipart()
# >>> m.field('search','searchish term')
# >>> m.file('greet','greet.txt','Hello multipart world!',{'Content-Type':'text/text'})
# >>> ct,body = m.get()
# >>> print ct
# multipart/form-data; boundary=----------AaB03x

# request = urllib2.Request(url='http://my.fake.server',
#     headers={'Content-Type':ct},
#     data=body)
# reply = urllib2.urlopen(request)
# print reply.read()

import mimetypes
 
class Part(object):
    '''
    Class holding a single part of the form.  You should never need to use
    this class directly; instead, use the factory methods in Multipart:
    field and file.
    '''
 
    # The boundary to use.  This is shamelessly taken from the standard.
    BOUNDARY = '----------AaB03x'
    CRLF = '\r\n'
    # Common headers.
    CONTENT_TYPE = 'Content-Type'
    CONTENT_DISPOSITION = 'Content-Disposition'
    # The default content type for parts.
    DEFAULT_CONTENT_TYPE = 'application/octet-stream'
 
    def __init__(self, name, filename, body, headers):
        '''
        Make a new part.  The part will have the given headers added initially.
 
        @param name: The part name.
        @type name: str
        @param filename: If this is a file, the name of the file.  Otherwise
                        None.
        @type filename: str
        @param body: The body of the part.
        @type body: str
        @param headers: Additional headers, or overrides, for this part.
                        You can override Content-Type here.
        @type headers: dict
        '''
        self._headers = headers.copy()
        self._name = name
        self._filename = filename
        self._body = body
        # We respect any content type passed in, but otherwise set it here.
        # We set the content disposition now, overwriting any prior value.
        if self._filename == None:
            self._headers[Part.CONTENT_DISPOSITION] = \
                ('form-data; name="%s"' % self._name)
            self._headers.setdefault(Part.CONTENT_TYPE,
                                     Part.DEFAULT_CONTENT_TYPE)
        else:
            self._headers[Part.CONTENT_DISPOSITION] = \
                ('form-data; name="%s"; filename="%s"' %
                 (self._name, self._filename))
            self._headers.setdefault(Part.CONTENT_TYPE,
                                     mimetypes.guess_type(filename)[0]
                                     or Part.DEFAULT_CONTENT_TYPE)
        return
 
    def get(self):
        '''
        Convert the part into a list of lines for output.  This includes
        the boundary lines, part header lines, and the part itself.  A
        blank line is included between the header and the body.
 
        @return: Lines of this part.
        @rtype: list
        '''
        lines = []
        lines.append('--' + Part.BOUNDARY)
        for (key, val) in self._headers.items():
            lines.append('%s: %s' % (key, val))
        lines.append('')
        lines.append(self._body)
        return lines
 
class Multipart(object):
    '''
    Encapsulate multipart form data.  To use this, make an instance and then
    add parts to it via the two methods (field and file).  When done, you can
    get the result via the get method.
 
    See http://www.w3.org/TR/html401/interact/forms.html#h-17.13.4.2 for
    details on multipart/form-data.
 
    Watch http://bugs.python.org/issue3244 to see if this is fixed in the
    Python libraries.
 
    @return: content type, body
    @rtype: tuple
    '''
 
    def __init__(self):
        self.parts = []
        return
 
    def field(self, name, value, headers={}):
        '''
        Create and append a field part.  This kind of part has a field name
        and value.
 
        @param name: The field name.
        @type name: str
        @param value: The field value.
        @type value: str
        @param headers: Headers to set in addition to disposition.
        @type headers: dict
        '''
        self.parts.append(Part(name, None, value, headers))
        return
 
    def file(self, name, filename, value, headers={}):
        '''
        Create and append a file part.  THis kind of part has a field name,
        a filename, and a value.
 
        @param name: The field name.
        @type name: str
        @param value: The field value.
        @type value: str
        @param headers: Headers to set in addition to disposition.
        @type headers: dict
        '''
        self.parts.append(Part(name, filename, value, headers))
        return
 
    def get(self):
        '''
        Get the multipart form data.  This returns the content type, which
        specifies the boundary marker, and also returns the body containing
        all parts and bondary markers.
 
        @return: content type, body
        @rtype: tuple
        '''
        all = []
        for part in self.parts:
            all += part.get()
        all.append('--' + Part.BOUNDARY + '--')
        all.append('')
        # We have to return the content type, since it specifies the boundary.
        content_type = 'multipart/form-data; boundary=%s' % Part.BOUNDARY
        return content_type, Part.CRLF.join(all)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import unittest
from StringIO import StringIO

from py_w3c.validators.html.validator import HTMLValidator
from py_w3c.exceptions import ValidationFault


TESTS_DIR = os.path.dirname(__file__)


class TestValidator(unittest.TestCase):
    def setUp(self):
        self.validator = HTMLValidator(charset='utf-8')

    def _fullpath(self, filename):
        '''
        returns full for file in tests directory.
        '''
        return os.path.join(TESTS_DIR, filename)

    def test_url_validation(self):
        # I know exactly there is no errors on datetostr.org
        self.validator.validate('http://datetostr.org')
        self.assertEqual(self.validator.errors, [])
        self.assertEqual(self.validator.warnings, [])

    def test_file_validation(self):
        with open(self._fullpath('file.html')) as f:
            self.validator.validate_file(f)
            self.assertEqual(len(self.validator.errors), 1)
            self.assertEqual(int(self.validator.errors[0].get('line')), 3)

    def test_validation_by_file_name(self):
        with open(self._fullpath('file.html')) as f:
            self.validator.validate_file(f.name)
            self.assertEqual(len(self.validator.errors), 1)
            self.assertEqual(int(self.validator.errors[0].get('line')), 3)

    def test_validation_by_file_with_unicode_name(self):
        with open(self._fullpath(u'мой-файл.html')) as f:
            self.validator.validate_file(f.name)
            self.assertEqual(len(self.validator.errors), 1)
            self.assertEqual(int(self.validator.errors[0].get('line')), 3)

    def test_in_memory_file_validation(self):
        HTML = '''<!DOCTYPE html>
            <html>
            <head bad-attr="i'm bad">
                <title>py_w3c test</title>
            </head>
                <body>
                    <h1>Hello py_w3c</h1>
                </body>
            </html>
        '''
        self.validator.validate_file(StringIO(HTML))
        self.assertEqual(len(self.validator.errors), 1)
        self.assertEqual(int(self.validator.errors[0].get('line')), 3)

    def test_fragment_validation(self):
        fragment = u'''<!DOCTYPE html>
            <html>
                <head>
                    <title>testing py_w3c</title>
                </head>
                <body>
                    <badtag>i'm bad</badtag>
                    <div>my div</div>
                </body>
            </html>
        '''.encode('utf-8')
        self.validator.validate_fragment(fragment)
        self.assertEqual(len(self.validator.errors), 1)
        self.assertEqual(int(self.validator.errors[0].get('line'),), 7)

    def test_passing_doctype_forces_validator_to_use_given_doctype(self):
        doctype = 'XHTML 1.0 Strict'
        val = HTMLValidator(doctype=doctype)
        # I know exactly there is no errors on datetostr.org
        val.validate('http://datetostr.org')
        self.assertTrue(doctype in val.result.doctype)

    def test_passing_charset_forces_validator_to_use_given_charset(self):
        val = HTMLValidator(charset='windows-1251')
        val.validate('http://datetostr.org')
        self.assertEqual(val.result.charset, 'windows-1251')

    def test_passing_wrong_charset_raises_ValidationFault_exception(self):
        val = HTMLValidator(charset='win-1251')
        self.assertRaises(ValidationFault, val.validate, 'http://datetostr.org')



from distutils.core import setup

from setuptools import find_packages

setup(
    name='py_w3c',
    version='0.1.2',
    author='Kazbek Byasov',
    author_email='nmb.ten@gmail.com',
    packages=find_packages(),
    url='http://pypi.python.org/pypi/py_w3c/',
    license='LICENSE.txt',
    description='W3C services for python.',
    long_description=open('README.txt').read(),
    install_requires=[],
    entry_points={
        'console_scripts':
            ['w3c_validate = py_w3c.validators.html.validator:main']
    }
)
